@@ -1,16 +1,17 @@
 from __future__ import annotations
 
+from dotenv import load_dotenv
 import os, asyncio
 from sqlalchemy import inspect
 from pathlib import Path
-from typing import Any, Optional, Sequence, Union, TYPE_CHECKING
+from typing import Any, Optional, Sequence, Union, TYPE_CHECKING, Set
 from alembic.config import Config
 
 from sqlalchemy import text
 from sqlalchemy.engine import URL, make_url
-from sqlalchemy.exc import OperationalError
+from sqlalchemy.exc import OperationalError, DBAPIError
 
-from .constants import ASYNC_DRIVER_HINT
+from .constants import ASYNC_DRIVER_HINT, DEFAULT_DB_ENV_VARS
 
 if TYPE_CHECKING:
     from sqlalchemy.engine import Engine as SyncEngine
@@ -44,6 +45,7 @@ def prepare_process_env(
           backward-compat only; prefer leaving it None.
     """
     root = Path(project_root).resolve()
+    load_dotenv(dotenv_path=root / ".env", override=False)
     os.environ.setdefault("SKIP_APP_INIT", "1")
 
     # Make <project>/src importable (env.py also handles this defensively)
@@ -121,54 +123,62 @@ def _compose_url_from_parts() -> Optional[str]:
 
 def get_database_url_from_env(
         required: bool = True,
-        env_vars: Sequence[str] = ("DATABASE_URL", "PRIVATE_DATABASE_URL", "DB_URL")
+        env_vars: Sequence[str] = DEFAULT_DB_ENV_VARS,
 ) -> Optional[str]:
     """
     Resolve the database connection string, with support for:
-      - Primary env vars: DATABASE_URL, PRIVATE_DATABASE_URL, DB_URL (in that order).
+      - Primary env vars (in order): DEFAULT_DB_ENV_VARS (e.g. DATABASE_URL, PRIVATE_DATABASE_URL, DB_URL).
       - Secret file pointers: <NAME>_FILE (reads file contents).
       - Well-known locations: DATABASE_URL_FILE, /run/secrets/database_url.
       - Composed from parts: DB_* (host, port, name, user, password, params).
-    This works for public or private networks—private DNS/socket addresses are just host strings.
+    When a value is found, it is also written back into os.environ["DATABASE_URL"] for downstream code.
     """
-    # 1) Direct envs
+    # Load .env without clobbering existing process env
+    load_dotenv(override=False)
+
+    # 1) Direct envs (and support "file:" or absolute path values)
     for key in env_vars:
         val = os.getenv(key)
         if val and val.strip():
             s = val.strip()
-            # Some platforms inject "file:" or path-like values—read them
+            # Some platforms inject a file-pointer-like value
             if s.startswith("file:"):
                 s = s[5:]
             if os.path.isabs(s) and Path(s).exists():
                 file_val = _read_secret_from_file(s)
                 if file_val:
+                    os.environ["DATABASE_URL"] = file_val
                     return file_val
+            os.environ["DATABASE_URL"] = s
             return s
 
-        # Companion NAME_FILE secret path
+        # Companion NAME_FILE secret path (e.g., DATABASE_URL_FILE)
         file_key = f"{key}_FILE"
         file_path = os.getenv(file_key)
         if file_path:
             file_val = _read_secret_from_file(file_path)
             if file_val:
+                os.environ["DATABASE_URL"] = file_val
                 return file_val
 
     # 2) Conventional secret envs
-    for file_key in ("DATABASE_URL_FILE",):
-        file_path = os.getenv(file_key)
-        if file_path:
-            file_val = _read_secret_from_file(file_path)
-            if file_val:
-                return file_val
+    file_path = os.getenv("DATABASE_URL_FILE")
+    if file_path:
+        file_val = _read_secret_from_file(file_path)
+        if file_val:
+            os.environ["DATABASE_URL"] = file_val
+            return file_val
 
     # 3) Docker/K8s default secret mount
     file_val = _read_secret_from_file("/run/secrets/database_url")
     if file_val:
+        os.environ["DATABASE_URL"] = file_val
         return file_val
 
-    # 4) Compose from parts (supports private DNS / unix sockets)
+    # 4) Compose from parts (DB_DIALECT/DB_DRIVER/DB_HOST/.../DB_PARAMS)
     composed = _compose_url_from_parts()
     if composed:
+        os.environ["DATABASE_URL"] = composed
         return composed
 
     if required:
@@ -178,6 +188,19 @@ def get_database_url_from_env(
         )
     return None
 
+def _ensure_timeout_default(u: URL) -> URL:
+    """
+    Ensure a conservative connection timeout is present for libpq-based drivers.
+    For psycopg/psycopg2, 'connect_timeout' is honored via the query string.
+    """
+    backend = (u.get_backend_name() or "").lower()
+    if backend not in ("postgresql", "postgres"):
+        return u
+    if "connect_timeout" in u.query:
+        return u
+    # Default 10s unless overridden
+    t = int(os.getenv("DB_CONNECT_TIMEOUT", "10"))
+    return u.set(query={**u.query, "connect_timeout": str(t)})
 
 # ---------- URL utilities ----------
 
@@ -198,23 +221,166 @@ def with_database(url: URL | str, database: Optional[str]) -> URL:
 
 # ---------- Engine creation ----------
 
-def build_engine(url: URL | str, echo: bool = False) -> Union[SyncEngine, AsyncEngineType]:
-    """Create a SQLAlchemy Engine or AsyncEngine based on the URL.
+def _coerce_to_async_url(url: str) -> str:
+    """Coerce common sync driver URLs to async-capable URLs.
 
-    - If the URL uses an async driver, returns an AsyncEngine (requires async extras installed).
-    - Otherwise returns a sync Engine.
-    - Pass `echo=True` to enable SQL echoing.
+    - postgresql:// or postgres://        -> postgresql+asyncpg://
+    - postgresql+psycopg2:// or +psycopg  -> postgresql+asyncpg://
+    - mysql:// or mysql+pymysql://        -> mysql+aiomysql://
+    - sqlite://                           -> sqlite+aiosqlite://
+    If already async (contains +asyncpg/+aiomysql/+aiosqlite), leave unchanged.
     """
+    low = url.lower()
+    if "+asyncpg" in low or "+aiomysql" in low or "+aiosqlite" in low:
+        return url
+    if low.startswith("postgresql+psycopg2://"):
+        return "postgresql+asyncpg://" + url.split("://", 1)[1]
+    if low.startswith("postgresql+psycopg://"):
+        return "postgresql+asyncpg://" + url.split("://", 1)[1]
+    if low.startswith("postgresql://"):
+        return "postgresql+asyncpg://" + url.split("://", 1)[1]
+    if low.startswith("postgres://"):
+        return "postgresql+asyncpg://" + url.split("://", 1)[1]
+    if low.startswith("mysql+pymysql://") or low.startswith("mysql://"):
+        return "mysql+aiomysql://" + url.split("://", 1)[1]
+    if low.startswith("sqlite://") and not low.startswith("sqlite+aiosqlite://"):
+        return "sqlite+aiosqlite://" + url.split("://", 1)[1]
+    return url
+
+def _is_mod_available(name: str) -> bool:
+    try:
+        __import__(name)
+        return True
+    except Exception:
+        return False
+
+def _coerce_sync_driver(u: URL) -> URL:
+    """
+    If URL has no explicit sync driver, pick one that’s available.
+    Postgres preference: psycopg (v3) → psycopg2
+    Optional override: DB_FORCE_DRIVER=psycopg|psycopg2
+    """
+    dn = (u.drivername or "").lower()
+    if "+" in dn:
+        return u
+
+    backend = (u.get_backend_name() or "").lower()
+    if backend in ("postgresql", "postgres"):
+        force = (os.getenv("DB_FORCE_DRIVER") or "").strip().lower()
+        if force in {"psycopg", "psycopg2"}:
+            return u.set(drivername=f"postgresql+{force}")
+
+        # prefer psycopg v3 if present; else psycopg2; else leave bare (SA default)
+        if _is_mod_available("psycopg"):
+            return u.set(drivername="postgresql+psycopg")
+        if _is_mod_available("psycopg2"):
+            return u.set(drivername="postgresql+psycopg2")
+        return u
+
+    if backend == "mysql":
+        if _is_mod_available("pymysql"):
+            return u.set(drivername="mysql+pymysql")
+        return u
+
+    if backend == "sqlite":
+        return u
+
+    if backend in ("mssql",):
+        if _is_mod_available("pyodbc"):
+            return u.set(drivername="mssql+pyodbc")
+        return u
+
+    return u
+
+
+def _coerce_pg_maintenance_driver(u: URL) -> URL:
+    """
+    Ensure the maintenance connection for Postgres uses a sync driver that is installed,
+    because ensure_database_exists() often needs a *sync* connection for CREATE DATABASE.
+    """
+    # If async → leave to async branch; this is for sync path only.
+    if "+" in (u.drivername or ""):
+        # If explicit async driver was given, leave it (caller decides).
+        return u
+    backend = (u.get_backend_name() or "").lower()
+    if backend in ("postgresql", "postgres"):
+        # prefer psycopg, then psycopg2
+        if _is_mod_available("psycopg"):
+            return u.set(drivername="postgresql+psycopg")
+        if _is_mod_available("psycopg2"):
+            return u.set(drivername="postgresql+psycopg2")
+    return u
+
+def _ensure_ssl_default(u: URL) -> URL:
+    backend = (u.get_backend_name() or "").lower()
+    if backend not in ("postgresql", "postgres"):
+        return u
+
+    driver = (u.drivername or "").lower()
+
+    # If any SSL hint already present, do nothing
+    if any(k in u.query for k in ("sslmode", "ssl", "sslrootcert", "sslcert", "sslkey")):
+        return u
+
+    # Allow env override; support both common spellings
+    mode_env = os.getenv("DB_SSLMODE_DEFAULT") or os.getenv("PGSSLMODE") or os.getenv("PGSSL_MODE")
+    mode = (mode_env or "").strip()
+
+    if "+asyncpg" in driver:
+        # asyncpg: use 'ssl=true' (SQLAlchemy forwards to asyncpg)
+        if "ssl" not in u.query:
+            return u.set(query={**u.query, "ssl": "true"})
+        return u
+    else:
+        # libpq-based drivers: use sslmode (default 'require' for hosted PG)
+        mode = mode or "require"
+        return u.set(query={**u.query, "sslmode": mode})
+
+def _ensure_ssl_default_async(u: URL) -> URL:
+    backend = (u.get_backend_name() or "").lower()
+    if backend in ("postgresql", "postgres"):
+        # asyncpg prefers 'ssl=true' via SQLAlchemy param; if already present, keep it
+        if any(k in u.query for k in ("ssl", "sslmode", "sslrootcert", "sslcert", "sslkey")):
+            return u
+        return u.set(query={**u.query, "ssl": "true"})
+    return u
+
+def build_engine(url: URL | str, echo: bool = False) -> Union[SyncEngine, AsyncEngineType]:
     u = make_url(url) if isinstance(url, str) else url
+    u = _ensure_ssl_default(u)
+    u = _ensure_timeout_default(u)   # <-- ADD THIS
+
+    # Async
     if is_async_url(u):
         create_async = _create_async_engine
         if create_async is None:
             raise RuntimeError("Async driver URL provided but SQLAlchemy async extras are not available.")
-        return create_async(u, echo=echo, pool_pre_ping=True)
+
+        connect_args: dict[str, Any] = {}
+        # asyncpg honors a top-level 'timeout' connect kwarg
+        if "+asyncpg" in (u.drivername or ""):
+            connect_args["timeout"] = int(os.getenv("DB_CONNECT_TIMEOUT", "10"))
+
+        kwargs: dict[str, Any] = {"echo": echo, "pool_pre_ping": True}
+        if connect_args:
+            kwargs["connect_args"] = connect_args
+        return create_async(u, **kwargs)
+
+    # Sync
+    u = _coerce_sync_driver(u)
     if _create_engine is None:
         raise RuntimeError("SQLAlchemy create_engine is not available in this environment.")
-    assert _create_engine is not None  # for type-checkers
-    return _create_engine(u, echo=echo, pool_pre_ping=True)
+
+    connect_args: dict[str, Any] = {}
+    dn = (u.drivername or "").lower()
+    if dn.startswith("postgresql+psycopg") and u.password:
+        connect_args["password"] = u.password
+
+    kwargs: dict[str, Any] = {"echo": echo, "pool_pre_ping": True}
+    if connect_args:
+        kwargs["connect_args"] = connect_args
+
+    return _create_engine(u, **kwargs)
 
 
 # ---------- Identifier quoting helpers ----------
@@ -247,38 +413,65 @@ async def _pg_create_database_async(url: URL) -> None:
     if not target_db:
         return
 
-    maintenance_url = with_database(url, "postgres")
+    u = _ensure_ssl_default(make_url(url))
+    maintenance_url = with_database(u, "postgres")
     engine: AsyncEngineType = build_engine(maintenance_url)  # type: ignore[assignment]
-    async with engine.begin() as conn:
-        exists = await conn.scalar(
-            text("SELECT 1 AS one FROM pg_database WHERE datname = :name"),
-            {"name": target_db},
-        )
-        if not exists:
-            quoted = _pg_quote_ident(target_db)
-            await conn.execution_options(isolation_level="AUTOCOMMIT").execute(
-                text(f'CREATE DATABASE "{quoted}"')
+
+    try:
+        async with engine.begin() as conn:
+            exists = await conn.scalar(
+                text("SELECT 1 AS one FROM pg_database WHERE datname = :name"),
+                {"name": target_db},
             )
-    await engine.dispose()
+            if not exists:
+                quoted = _pg_quote_ident(target_db)
+                await conn.execution_options(isolation_level="AUTOCOMMIT").execute(
+                    text(f'CREATE DATABASE "{quoted}"')
+                )
+    except DBAPIError as e:
+        if "permission denied" in str(e).lower():
+            pass
+        else:
+            raise
+    finally:
+        await engine.dispose()
 
 
 def _pg_create_database_sync(url: URL) -> None:
     target_db = url.database
     if not target_db:
         return
-    maintenance_url = with_database(make_url(url), "postgres")
-    engine: SyncEngine = build_engine(maintenance_url)  # type: ignore[assignment]
-    with engine.begin() as conn:
-        exists = conn.scalar(
-            text("SELECT 1 AS one FROM pg_database WHERE datname = :name"),
-            {"name": target_db},
-        )
-        if not exists:
-            quoted = _pg_quote_ident(target_db)
-            conn.execution_options(isolation_level="AUTOCOMMIT").execute(
-                text(f'CREATE DATABASE "{quoted}"')
-            )
-    engine.dispose()
+
+    # Build a maintenance URL pointing at the 'postgres' DB
+    u = _ensure_ssl_default(make_url(url))
+    maintenance_url = with_database(u, "postgres")
+
+    # Make sure it has an installed *sync* driver (psycopg or psycopg2)
+    maintenance_url = _coerce_pg_maintenance_driver(maintenance_url)
+
+    # Try connecting to 'postgres'; if that fails (some hosts restrict it), try 'template1'
+    try:
+        engine: SyncEngine = build_engine(maintenance_url)  # type: ignore[assignment]
+    except Exception:
+        alt_url = with_database(maintenance_url, "template1")
+        engine = build_engine(alt_url)  # type: ignore[assignment]
+
+    try:
+        with engine.begin() as conn:
+            exists = conn.scalar(text("SELECT 1 FROM pg_database WHERE datname = :name"), {"name": target_db})
+            if not exists:
+                quoted = _pg_quote_ident(target_db)
+                conn.execution_options(isolation_level="AUTOCOMMIT").execute(
+                    text(f'CREATE DATABASE "{quoted}"')
+                )
+    except DBAPIError as e:
+        # If permission error, log and continue (DB likely pre-provisioned on the host)
+        if "permission denied" in str(e).lower():
+            pass
+        else:
+            raise
+    finally:
+        engine.dispose()
 
 
 async def _mysql_create_database_async(url: URL) -> None:
@@ -471,6 +664,8 @@ async def _redshift_create_database_async(url: URL) -> None:
 # ---------- Entry: ensure database ----------
 
 def ensure_database_exists(url: URL | str) -> None:
+    if url is None:
+        raise RuntimeError("ensure_database_exists: database URL is required but None was provided.")
     u = make_url(url) if isinstance(url, str) else url
     backend = (u.get_backend_name() or "").lower()
 
@@ -511,15 +706,15 @@ def ensure_database_exists(url: URL | str) -> None:
         raise RuntimeError(f"Failed to connect to database: {exc}") from exc
 
 def repair_alembic_state_if_needed(cfg: Config) -> None:
-    """If DB points to a non-existent local revision, reset to base."""
+    """If DB points to a non-existent local revision, reset to base (drop alembic_version)."""
     db_url = cfg.get_main_option("sqlalchemy.url") or os.getenv("DATABASE_URL")
     if not db_url:
         return
 
-    # Collect local revision IDs
+    # Gather local revision ids from versions/
     script_location = Path(cfg.get_main_option("script_location"))
     versions_dir = script_location / "versions"
-    local_ids = set()
+    local_ids: Set[str] = set()
     if versions_dir.exists():
         for p in versions_dir.glob("*.py"):
             try:
@@ -533,23 +728,93 @@ def repair_alembic_state_if_needed(cfg: Config) -> None:
                     local_ids.add(rid)
                     break
 
-    eng = build_engine(db_url)
-    try:
-        with eng.begin() as c:
-            insp = inspect(c)
-            has_version_tbl = insp.has_table("alembic_version")
+    url_obj = make_url(db_url)
+    if is_async_url(url_obj):
+        async def _run() -> None:
+            eng = build_engine(url_obj)  # AsyncEngine
+            try:
+                async with eng.begin() as conn:
+                    # Do sync-y inspector / SQL via run_sync
+                    def _check_and_maybe_drop(sync_conn):
+                        insp = inspect(sync_conn)
+                        if not insp.has_table("alembic_version"):
+                            return
+                        rows = list(sync_conn.execute(text("SELECT version_num FROM alembic_version")).fetchall())
+                        missing = any((ver not in local_ids) for (ver,) in rows)
+                        if missing:
+                            sync_conn.execute(text("DROP TABLE IF EXISTS alembic_version"))
+                    await conn.run_sync(_check_and_maybe_drop)
+            finally:
+                await eng.dispose()
+        asyncio.run(_run())
+    else:
+        eng = build_engine(url_obj)  # sync Engine
+        try:
+            with eng.begin() as c:
+                insp = inspect(c)
+                if not insp.has_table("alembic_version"):
+                    return
+                rows = list(c.execute(text("SELECT version_num FROM alembic_version")).fetchall())
+                missing = any((ver not in local_ids) for (ver,) in rows)
+                if missing:
+                    c.execute(text("DROP TABLE IF EXISTS alembic_version"))
+        finally:
+            eng.dispose()
 
-            rows = []
-            if has_version_tbl:
-                rows = c.execute(text("SELECT version_num FROM alembic_version")).fetchall()
+def render_env_py(packages: Sequence[str], *, async_db: bool | None = None) -> str:
+    """Render Alembic env.py content from packaged templates.
 
-            missing = any((ver not in local_ids) for (ver,) in rows)
+    - If async_db is None, detect from DATABASE_URL; default to sync if unknown.
+    """
+    import importlib.resources as pkg
+    from sqlalchemy.engine import make_url as _make_url
+
+    if async_db is None:
+        try:
+            db_url = get_database_url_from_env(required=False)
+            async_db = bool(db_url and is_async_url(_make_url(db_url)))
+        except Exception:
+            async_db = False
+
+    pkg_list = ", ".join(repr(p) for p in packages)
+    tmpl_root = pkg.files("svc_infra.db.templates.setup")
+    name = "env_async.py.tmpl" if async_db else "env_sync.py.tmpl"
+    txt = tmpl_root.joinpath(name).read_text(encoding="utf-8")
+    return txt.replace("__PACKAGES_LIST__", pkg_list)
+
+
+def build_alembic_config(project_root: Path | str, *, script_location: str = "migrations") -> Config:
+    from alembic.config import Config as _Config
+    root = Path(project_root).resolve()
+    cfg_path = root / "alembic.ini"
+    cfg = _Config(str(cfg_path)) if cfg_path.exists() else _Config()
+    cfg.set_main_option("script_location", str((root / script_location).resolve()))
+
+    env_db_url = os.getenv("DATABASE_URL", "").strip()
+    if env_db_url:
+        u = make_url(env_db_url)
+        u = _ensure_ssl_default(u)
+        if not is_async_url(u):
+            u = _coerce_sync_driver(u)
+        cfg.set_main_option("sqlalchemy.url", u.render_as_string(hide_password=False))
+
+    # >>> ADD THIS GUARD <<<
+    if not cfg.get_main_option("sqlalchemy.url"):
+        raise RuntimeError(
+            "No SQLAlchemy URL resolved. Pass `database_url` to the calling function "
+            "or set DATABASE_URL in the environment."
+        )
+
+    cfg.set_main_option("path_separator", "os")
+    cfg.set_main_option("prepend_sys_path", str(root))
+    return cfg
+
+
+def ensure_db_at_head(cfg: Config) -> None:
+    """Idempotently bring the database to head; safe if already current."""
+    from alembic import command as _command
+    _command.upgrade(cfg, "head")
 
-            if missing:
-                # safest reset: drop version table, caller will autogen + upgrade
-                c.execute(text("DROP TABLE IF EXISTS alembic_version"))
-    finally:
-        eng.dispose()
 
 __all__ = [
     # env helpers
@@ -559,4 +824,8 @@ __all__ = [
     # engines and db bootstrap
     "build_engine",
     "ensure_database_exists",
+    # setup helpers
+    "render_env_py",
+    "build_alembic_config",
+    "ensure_db_at_head",
 ]
